import json, urllib2
from front_end.models import *

class DailyMile(object):
	
	def __init__(self, username, startTime, endTime, currentIteration):
		self.username = username
		self.startTime = startTime
		self.endTime = endTime
		self.current_iteration = currentIteration

	def add(self, friend_obj, wType, dist, dur, mesg, theTime, currentIteration, dailymileID, felt):
		""" Will add a workout to the database. """
		try:
			distance_norm = dist['value']
		except:
			distance_norm = 0

		try:
			workoutType = Workout_Type.objects.get(name=wType)
			w = Workout(
				friend=friend_obj,
				workout_type=workoutType,
				distance=distance_norm,
				duration=dur,
				iteration=currentIteration,
				dailymile_id=dailymileID,
				felt=felt,
				message=mesg,
				date=theTime[0:10])
			w.save()
		except:
			print "Error, could not find workout type of %s" % wType

	def data_pull(self, url = False):
		""" Pulls data from specified url """

		if url is False:
			return False
		else:   
			try:
				data = json.loads(urllib2.urlopen(url).read())
				return data
			except urllib2.HTTPError, e:
				self.error("HTTP Error: %d" % e.code)
				return False
			except urllib2.URLError, e:
				self.error("Network Error: $s" % e.reason.args[1])
				return False

	def fetch_data(self, fetch_new_data, friend):
		""" Gets every friend's data from DailyMile """

		if not fetch_new_data:
		 	return False

		Workout.objects.filter(iteration=self.current_iteration, friend=friend).delete() # Truncate the table.
	
		friend_data = self.get_friend_data(friend.user_name)
		for d in friend_data:
			# Edge case for when someone doesn't move forward, we still need a number.
			try:
				distance = d[1]['distance']
			except:
				distance = 0.0

			# Edge case for when someone doesn't enter their time.
			try:
				duration = d[1]['duration']
			except:
				duration = 0

			# Edge case for when someone doesn't enter how they felt.
			try:
				felt = d[1]['felt']
			except:
				felt = 'No Data'

			self.add(friend, d[1]['activity_type'], distance, duration, d[2], d[3], self.current_iteration, d[0], felt)


	def get_friend_data(self, username):
		""" Gets a specified friend's data from DailyMile """

		allData = []
		for i in range(1,50):
			url = "https://api.dailymile.com/people/%s/entries.json?since=%d&until=%d&page=%d" % (username, self.startTime, self.endTime, i)
			data = self.data_pull(url)

			for e in data['entries']:
				message = ""
				try:
					message = e['message']
				except:
					pass

				try:
					#print "Workout found for %s" % username
					allData.append([e['id'],e['workout'],message, e['at']])
				except:
					# Not a workout.
					#print "Not a workout."
					#print e
					pass

			if len(data['entries']) < 20:
				break
		
		return allData
	

	def update_friends(self, fetch_new_data):
		""" Adds or updates friend's information """

		if not fetch_new_data:
			return False

		url = "https://api.dailymile.com/people/%s/friends.json" % self.username
		friendData = self.data_pull(url)
		if(friendData):
			#print "Fetching friend updates..."
			for fData in friendData['friends']:
				f = False
				try:
					f = Friend.objects.get(user_name=fData['username'])
					f.user_name = fData['username']
					f.display_name = fData['display_name']
					f.url = fData['url']
					f.photo_url = fData['photo_url']
				except Friend.DoesNotExist:
					#print "Friend not found... adding."
					pass

				if not f:
					f = Friend(
						user_name=fData['username'],
						display_name=fData['display_name'],
						url=fData['url'],
						photo_url=fData['photo_url'])

				f.save()
		else:
			self.error("There was an error retrieving %s's friends from the server" % username)
	