from django.db import models
import math, datetime, calendar

WORKOUT_TYPE_CHOICES = (
        ("Distance","Distance"),
        ("Duration","Duration")
)

class Friend(models.Model):
        user_name = models.CharField(max_length=75)
        display_name = models.CharField(max_length=100)
        first_name = models.CharField(max_length=50)
        last_name = models.CharField(max_length=100)
        url = models.CharField(max_length=200)
        photo_url = models.CharField(max_length=200)
        large_photo_url = models.CharField(max_length=200)
        exclude = models.BooleanField()

        def __unicode__(self):
                return self.first_name + " " + self.last_name + " (" + self.user_name + ")"

class Iteration(models.Model):
        start_time = models.DateTimeField()
        end_time = models.DateTimeField()
        description = models.CharField(max_length=200)
        current = models.BooleanField()

        def __unicode__(self):
                return self.description

class Last_Update(models.Model):
        last_update_time = models.DateTimeField(auto_now=True)

        def __unicode__(self):
                return self.last_update_time

class WorkoutManager(models.Manager):
        def get_category_breakdown(self, the_friend=False):
                ''' Returns a list of tuples with the category and percentage '''
                current_iteration = Iteration.objects.get(current=True)
                if the_friend:
                        workout_types = self.filter(friend=the_friend, iteration=current_iteration).values('workout_type').distinct()
                        total_workouts = self.filter(friend=the_friend, iteration=current_iteration).count()
                else:
                        workout_types = self.filter(iteration=current_iteration).values('workout_type').distinct()
                        total_workouts = self.filter(iteration=current_iteration).count()

                workout_percentages = []
                for workout_type in workout_types:
                        w_type = Workout_Type.objects.get(id=workout_type['workout_type'])
                        if the_friend:
                                workout_count = self.filter(workout_type=w_type, friend=the_friend, iteration=current_iteration).count()
                        else:
                                workout_count = self.filter(workout_type=w_type, iteration=current_iteration).count()

                        workout_percentages.append((w_type.name, round((float(workout_count)/float(total_workouts)*100),2), workout_count))

                return workout_percentages

        def get_data_by_workout_type(self, the_workout_type, the_friend=False):
                ''' Get a list of all workouts in the given workout type, and filter by friend if one given.'''
                current_iteration = Iteration.objects.get(current=True)
                if the_friend:
                        workouts = self.filter(workout_type=the_workout_type, friend=the_friend, iteration=current_iteration)
                else:
                        workouts = self.filter(workout_type=the_workout_type, iteration=current_iteration)
                return workouts

        def get_all_distance_duration(self, the_workout_type, the_friend=False):
                current_iteration = Iteration.objects.get(current=True)
                total = 0.0
                if the_friend:
                        workouts = self.filter(workout_type=the_workout_type, friend=the_friend, iteration=current_iteration)   
                else:
                        workouts = self.filter(workout_type=the_workout_type, iteration=current_iteration)
                for w in workouts:
                        if w.workout_type.type == "Distance":
                                total += float(w.distance)
                        else:
                                total += w.duration

                if the_workout_type.type == "Distance":
                        return total
                else:
                        return total / 60

        def get_points_per_workout_type(self, the_workout_type, the_friend=False):
                current_iteration = Iteration.objects.get(current=True)
                if the_friend:
                        workouts = self.filter(workout_type=the_workout_type, friend=the_friend, iteration=current_iteration)
                else:
                        workouts = self.filter(workout_type=the_workout_type, iteration=current_iteration)

                total_points = 0.0
                for w in workouts:
                        total_points = total_points + w.get_points()

                return total_points

        def get_points_for_all(self, workout_type=False):                
                all_points = []
                point_total = 0
                friends = Friend.objects.filter(exclude=0)
                current_iteration = Iteration.objects.get(current=True)
                for f in friends:
                        if workout_type:
                                workouts = Workout.objects.filter(friend=f, workout_type=workout_type, iteration=current_iteration)
                        else:
                                workouts = Workout.objects.filter(friend=f, iteration=current_iteration)
                        points = 0
                        for w in workouts:
                                points = points + w.get_points()

                        if points > 0:
                                all_points.append({
                                        'friend' : f,
                                        'points' : math.ceil(points * 100)/100
                                })
                                point_total += math.ceil(points*100)/100

                return (all_points, point_total)

        def get_sentiment_data(self, the_friend=False, the_workout_type=False):
                current_iteration = Iteration.objects.get(current=True)
                if the_friend:
                        if the_workout_type:
                                sentiment_types = self.filter(friend=the_friend, workout_type=the_workout_type, iteration=current_iteration).values('felt').distinct()
                                total_workouts = self.filter(friend=the_friend, workout_type=the_workout_type, iteration=current_iteration).count()
                        else:
                                sentiment_types = self.filter(friend=the_friend, iteration=current_iteration).values('felt').distinct()
                                total_workouts = self.filter(friend=the_friend, iteration=current_iteration).count()
                else:
                        if the_workout_type:
                                sentiment_types = self.filter(workout_type=the_workout_type, iteration=current_iteration).values('felt').distinct()
                                total_workouts = self.filter(workout_type=the_workout_type, iteration=current_iteration).count()
                        else:
                                sentiment_types = self.filter(iteration=current_iteration).values('felt').distinct()
                                total_workouts = self.objects.filter(iteration=current_iteration).count()

                sentiment_percentages = []
                for sentiment_type in sentiment_types:
                        if the_friend:
                                if the_workout_type:
                                        sentiment_count = self.filter(felt=sentiment_type['felt'], friend=the_friend, workout_type=the_workout_type, iteration=current_iteration).count()
                                else:
                                        sentiment_count = self.filter(felt=sentiment_type['felt'], friend=the_friend, iteration=current_iteration).count()
                        else:
                                if the_workout_type:
                                        sentiment_count = self.filter(felt=sentiment_type['felt'], workout_type=the_workout_type, iteration=current_iteration).count()
                                else:
                                        sentiment_count = self.filter(felt=sentiment_type['felt'], iteration=current_iteration).count()

                        
                        sentiment_percentages.append((sentiment_type['felt'], round((float(sentiment_count)/float(total_workouts)*100),2), sentiment_count))

                return sentiment_percentages

        def get_workout_days_of_week(self, the_friend=False, the_workout_type=False):
                current_iteration = Iteration.objects.get(current=True)
                if the_friend:
                        if the_workout_type:
                                workouts = self.filter(friend=the_friend, workout_type=the_workout_type, iteration=current_iteration)
                        else:
                                workouts = self.filter(friend=the_friend, iteration=current_iteration)
                else:
                        if the_workout_type:
                                workouts = self.filter(workout_type=the_workout_type, iteration=current_iteration)
                        else:
                                workouts = self.filter(iteration=current_iteration)

                days_of_week = {
                        "Sunday" : 0,
                        "Monday" : 0,
                        "Tuesday" : 0,
                        "Wednesday" : 0,
                        "Thursday" : 0,
                        "Friday" : 0,
                        "Saturday" : 0,
                }
                for w in workouts:
                        days_of_week[w.get_day_of_week()] += 1

                return days_of_week

        def get_workouts_done_by_month(self, the_friend=False, the_workout_type=False):
                current_iteration = Iteration.objects.get(current=True)
                if the_friend:
                        if not the_workout_type:
                                months = Workout.objects.filter(friend=the_friend, iteration=current_iteration).dates('date', 'month', order='ASC')
                        else:
                                months = Workout.objects.filter(friend=the_friend, workout_type=the_workout_type, iteration=current_iteration).dates('date', 'month', order='ASC')
                else:
                        if not the_workout_type:
                                months = Workout.objects.filter(iteration=current_iteration).dates('date', 'month', order='ASC')
                        else:
                                months = Workout.objects.filter(workout_type=the_workout_type, iteration=current_iteration).dates('date', 'month', order='ASC')
                
                month_data = []
                for m in months:
                        number_of_workouts = 0
                        if the_friend:
                                if not the_workout_type:
                                        workouts = Workout.objects.filter(friend=the_friend, iteration=current_iteration)
                                else:
                                        workouts = Workout.objects.filter(friend=the_friend, workout_type=the_workout_type, iteration=current_iteration)   
                                for w in workouts:
                                        if w.date.month == m.month:
                                                number_of_workouts += 1;
                        else:
                                if not the_workout_type:
                                        workouts = Workout.objects.filter(iteration=current_iteration)
                                else:
                                        workouts = Workout.objects.filter(workout_type=the_workout_type, iteration=current_iteration)
                                
                                for w in workouts:
                                        if w.date.month == m.month:
                                                number_of_workouts += 1;

                        month_data.append((calendar.month_name[m.month], number_of_workouts))

                return month_data

class Workout(models.Model):
        friend = models.ForeignKey('Friend',db_index=True)
        date = models.DateField()
        workout_type = models.ForeignKey('Workout_Type',db_index=True)
        distance = models.CharField(max_length=50)
        duration = models.IntegerField()
        iteration = models.ForeignKey('Iteration',db_index=True)
        dailymile_id = models.IntegerField()
        felt = models.CharField(max_length=30)
        message = models.TextField()
        objects = WorkoutManager()

        def get_day_of_week(self):
                weekdays = []
                weekdays.append('Sunday')
                weekdays.append('Monday')
                weekdays.append('Tuesday')
                weekdays.append('Wednesday')
                weekdays.append('Thursday')
                weekdays.append('Friday')
                weekdays.append('Saturday')
                return weekdays[self.date.weekday()]

        def get_points(self):
                points = 0
                if self.workout_type.type == "Distance":
                        points = float(self.distance) * float(self.workout_type.point_value)
                elif self.workout_type.type == "Duration":
                        points = float(self.duration/60) / float(self.workout_type.point_value)

                return points

        def __unicode__(self):
                return "%s %s (%s)" % (self.friend.first_name, self.friend.last_name, self.workout_type.name)

class Workout_Type(models.Model):
	name = models.CharField(max_length=100)
        type = models.CharField(max_length=25, choices=WORKOUT_TYPE_CHOICES)
        point_value = models.DecimalField(max_digits=10,decimal_places=8)
        description = models.CharField(max_length=100)

	def __unicode__(self):
		return self.name