var battle = (function($, undefined) {
	var self = this;
	var left_data = false;
	var right_data = false;
	var template_target = false;
	var template = false;
	var url = "/battle/data/";

	// Make sure the constructor gets called when the page is ready.
	$(document).ready(function() {
		init();
	});

	// Calculate the winner of a category.
	// Usage: calculate_winner(1.2,1.4,".total-points")
	var calculate_winner = function(left, right, target) {
		
		//Unfortunately toFixed() changes the number to a string.
		try {
			left = parseFloat(left);
			right = parseFloat(right);
		} catch (err) {}
				
		var left_col = $(target + " .bleft");
		var right_col = $(target + " .bright");
		var middle_col = $(target + " .bmiddle");
		var status = "";
		remove_status(left_col, right_col, middle_col);

		if(left > right) {
			left_col.addClass("win");
			right_col.addClass("loss");
			middle_col.addClass("left-win");
			status = "left";
		} else if(right > left) {
			left_col.addClass("loss");
			right_col.addClass("win");
			middle_col.addClass("right-win");
			status = "right";
		} else if(left == right) {
			left_col.addClass("tie");
			right_col.addClass("tie");
			middle_col.addClass("tie");
			status = "tie";
		}
		return status;
	};

	// Determine the overall winner and then draw that to the screen.
	var determine_overall_winner = function(results) {
		var left_count = 0;
		var right_count = 0;
		for(var i = 0; i < results.length; i++) {
			if(results[i] == "left") {
				left_count++;
			} else if(results[i] == "right") {
				right_count++;
			}
		}

		var message;
		if(left_count == right_count) { 
			message = "Tie!";
		} else if(left_count > right_count) {
			message = "Winner " + $("#friend-left option:selected").text() + "!";
		} else {
			message = "Winner " + $("#friend-right option:selected").text() + "!";
		}

		$(".winner").html(message).fadeIn();	
	};

	// Fetch the data from the server and store it locally.
	var get_data = function() {
		var selected = $(this);
		if(selected.val() !== "select") {
			var target = selected.attr("data-target");
			$.getJSON(url, {
				"friendID" : selected.val()
			}, function(data) {
				if(target === "#friend-data-left") {
					left_data = data;
				} else {
					right_data = data;
				}
				start_battle();
			});
		}
	};

	// Add event handlers to the select boxes.
	var init = function() {
		$("#friend-left,#friend-right").change(get_data);
		template = Handlebars.compile($("#battle-template").html());
		template_target = $("#battle-data");
	};

	// Remove the win/loss status from the columns.
	var remove_status = function(left, right, middle) {
		left.removeClass("win").removeClass("loss").removeClass("tie");
		right.removeClass("win").removeClass("loss").removeClass("tie");
		middle.removeClass("left-win").removeClass("right-win").removeClass("tie");
	};

	// Start rendering the battle, but only if both sides have data.
	var start_battle = function() {
		if(left_data !== false && right_data !== false) {

			// Hide the previous win/loss/tie message.
			$(".winner").fadeOut().html("");

			// Build and merge the context.
			var left_context = {
				total_points_left : left_data.points.toFixed(2),
				number_of_workouts_left : left_data.total_workouts.toFixed(2),
				average_points_per_workout_left : left_data.average_points_per_workout.toFixed(2),
				distance_traveled_left : left_data.distance.toFixed(2),
				time_working_out_left : left_data.time.toFixed(2),
				weekend_warrior_points_left: left_data.total_weekends.toFixed(2),
				weekday_wonder_points_left: left_data.total_weekdays
			};
			var right_context = {
				total_points_right : right_data.points.toFixed(2),
				number_of_workouts_right : right_data.total_workouts.toFixed(2),
				average_points_per_workout_right : right_data.average_points_per_workout.toFixed(2),
				distance_traveled_right : right_data.distance.toFixed(2),
				time_working_out_right : right_data.time.toFixed(2),
				weekend_warrior_points_right: right_data.total_weekends.toFixed(2),
				weekday_wonder_points_right: right_data.total_weekdays
			};
			var context = $.extend({}, left_context, right_context);

			// Template the data.
			var html = template(context);
			template_target.html(html);

			// Determine winners
			var results = [];
			results.push(calculate_winner(left_context.total_points_left, right_context.total_points_right, ".total-points"));
			results.push(calculate_winner(left_context.number_of_workouts_left, right_context.number_of_workouts_right, ".number-of-workouts"));
			results.push(calculate_winner(left_context.average_points_per_workout_left, right_context.average_points_per_workout_right, ".average-points-per-workout"));
			results.push(calculate_winner(left_context.distance_traveled_left, right_context.distance_traveled_right, ".distance-traveled"));
			results.push(calculate_winner(left_context.time_working_out_left, right_context.time_working_out_right, ".time-working-out"));
			results.push(calculate_winner(left_context.weekend_warrior_points_left, right_context.weekend_warrior_points_right, ".weekend-warrior"));
			results.push(calculate_winner(left_context.weekday_wonder_points_left, right_context.weekday_wonder_points_right, ".weekday-wonder"));
			
			// Now count the occurrance of each item. and set the winner message!
			determine_overall_winner(results);
		}
	};

	return self;
}(jQuery, undefined));