from django.shortcuts import render_to_response, redirect
from django.http import Http404, HttpResponse
from django.views.decorators.cache import cache_page, never_cache
from django.template import RequestContext
from front_end.models import *
from annoying.decorators import ajax_request
from utility import Utility
import datetime, math, logging

def battle(request):
	return render_to_response("battle.html",{
		"friends" : Friend.objects.filter(exclude=0).order_by('first_name')
	},context_instance=RequestContext(request))

@ajax_request
def battle_ajax(request):
	current_iteration = Iteration.objects.get(current=True)

	# Grab the friend.
	friend_id = request.GET['friendID']
	friend = Friend.objects.get(pk=friend_id)

	workouts = Workout.objects.filter(friend=friend, iteration=current_iteration)
	total_points = 0.0
	total_duration = 0
	total_distance = 0.0
	total_workouts = workouts.count()
	total_weekdays = 0
	total_weekends = 0
	for w in workouts:
		total_points += w.get_points()
		total_duration += w.duration
		total_distance += float(w.distance)
		day_of_week = w.get_day_of_week()
		if(day_of_week == "Sunday" or day_of_week == "Saturday"):
			total_weekends += 1
		else:
			total_weekdays += 1

	if total_workouts == 0:
		average_points_per_workout = 0
	else:
		average_points_per_workout = total_points/total_workouts

	return {
		"points" : total_points,
		"time" : total_duration,
		"distance" : total_distance,
		"total_workouts" : workouts.count(),
		"average_points_per_workout" : average_points_per_workout,
		"total_weekdays" : total_weekdays,
		"total_weekends" : total_weekends
	}

def category(request):
	types = Workout_Type.objects.filter()
	type_points = []
	for t in types:
		type_points.append({
			"workout_type" : t.name,
			"points" : Workout.objects.get_points_per_workout_type(t)
		})

	return render_to_response("category.html", {
		'workout_types' : Workout_Type.objects.all().order_by('name'),
		'category_data' : Workout.objects.get_category_breakdown(),
		'type_points' : type_points
	},context_instance=RequestContext(request))

def category_specific(request, category):
	current_iteration = Iteration.objects.get(current=True)

	workout_type_name = category.replace("-", " ")
	try:
		current_workout_type = Workout_Type.objects.filter(name__iexact=workout_type_name)[0]
	except:
		raise Http404

	try:
		# Normally I would have called these below, but I needed them for the average.
		total_workouts = Workout.objects.filter(workout_type=current_workout_type, iteration=current_iteration).count()
		total_distance_duration = Workout.objects.get_all_distance_duration(current_workout_type)
		average = total_distance_duration / total_workouts


		# Get the duration / distance per person & the average workout distance/duration.
		friends = Friend.objects.filter(exclude=0).order_by('first_name')
		distance_duration_per_person = []
		for f in friends:
			friend_total_workouts = Workout.objects.filter(workout_type=current_workout_type, friend=f, iteration=current_iteration).count()
			value = Workout.objects.get_all_distance_duration(current_workout_type,f)

			if value > 0 and friend_total_workouts > 0:
				distance_duration_per_person.append({
					"friend" : f,
					"value" : value,
					"average" : value / friend_total_workouts
				})
			else:
				distance_duration_per_person.append({
					"friend" : f,
					"value" : value,
					"average" : 0
				})

		# Get notes
		workouts = Workout.objects.filter(workout_type=current_workout_type, iteration=current_iteration).order_by('date')
		notes = []
		for w in workouts:
			if w.message.strip() is not "":
				notes.append(w.friend.display_name + " - " + w.message);

		points_per_person, all_points = Workout.objects.get_points_for_all(current_workout_type)

		return render_to_response("category_specific.html", {
			"workout_type" : current_workout_type,
			"total_points" : Workout.objects.get_points_per_workout_type(current_workout_type),
			"total_workouts" : total_workouts,
			"total_distance_duration" : total_distance_duration,
			"points_per_person" : points_per_person,
			"average" : average,
			"distance_duration_per_person" : distance_duration_per_person,
			"workouts_by_month" : Workout.objects.get_workouts_done_by_month(False,current_workout_type),
			"workouts_by_week" : Workout.objects.get_workout_days_of_week(False, current_workout_type),
			"sentiment_data" : Workout.objects.get_sentiment_data(False, current_workout_type),
			"notes" : notes

		},context_instance=RequestContext(request))
	except:
		return render_to_response("no_data.html",context_instance=RequestContext(request))

def home(request):
	''' The home page.  Displays overall point graph and general announcements'''
	from operator import itemgetter
	updates = Last_Update.objects.all()
	for u in updates:
		update_time = u.last_update_time

	unsorted_points, total_points = Workout.objects.get_points_for_all()
	all_points = sorted(unsorted_points, key=itemgetter('points'), reverse=True)

	return render_to_response("home.html", {
		"all_points" : all_points,
		"total_points" : total_points,
		"last_updated" : update_time,
		"iteration" : Iteration.objects.get(current=True)
	},context_instance=RequestContext(request))

def people(request):
	''' Display a list of all people. '''
	return render_to_response("people.html", {
		"friends" : Friend.objects.filter(exclude=0).order_by('first_name')
	},context_instance=RequestContext(request))

def people_specific(request, person):
	''' Display a specific person, if they exist.'''

	current_iteration = Iteration.objects.get(current=True)

	u = Utility()
	first_name, last_name = u.split_name(person)
	try:
		current_friend = Friend.objects.filter(first_name=first_name,last_name=last_name)[0]
	except:
		raise Http404

	# Get all workouts for person.
	all_workouts = Workout.objects.filter(friend=current_friend, iteration=current_iteration).order_by('-date')

	# Get number of workouts done by day of week.
	day_of_week_data = Workout.objects.get_workout_days_of_week(the_friend=current_friend)

	# Get the number of workouts done by month.
	workouts_by_month = Workout.objects.get_workouts_done_by_month(the_friend=current_friend)

	# Sentiment data
	sentiment_data = Workout.objects.get_sentiment_data(the_friend=current_friend)

	# Get the total points
	total_points = 0
	for w in all_workouts:
		total_points += w.get_points()

	person_data = {
		"friend" : current_friend,
		"total_points" : round(total_points,2),
		"all_workouts" : all_workouts,
		"workout_types" : Workout.objects.get_category_breakdown(current_friend),
		"day_of_week_data" : day_of_week_data,
		"workouts_by_month" : workouts_by_month,
		"sentiment_data" : sentiment_data
	}
	
	return render_to_response("people_specific.html", person_data, context_instance=RequestContext(request))

@never_cache
def update(request, key, id):

	if (key == "13123adfasdfasdf123123asdfasdfoiuoiu098098asfdiu") or (key == "13123adfasdfasdf123123asdfasdfoiuoiu098098asfdiu-all"):
		
		import time
		from dailyMile import DailyMile
		from django.http import HttpResponseRedirect
		from django.core.cache import cache

		# Do you want to fetch new data?
		fetch_new_data = True

		# Decide what interations data is going to get pulled from.
		if key == "13123adfasdfasdf123123asdfasdfoiuoiu098098asfdiu-all":
			iterations = Iteration.objects.all()
		else:
			iterations = Iteration.objects.filter(current=1)

		# Get the current iteration start and end times.
		for currentIteration in iterations:
			startTime = int(time.mktime(currentIteration.start_time.timetuple()))
			endTime = int(time.mktime(currentIteration.end_time.timetuple()))

			# Get me some data!
			f = Friend.objects.get(pk=id)
			dm = DailyMile("vital101", startTime, endTime, currentIteration)
			dm.update_friends(fetch_new_data)
			dm.fetch_data(fetch_new_data, f)

		# Set the update time.
		time_update = Last_Update()
		time_update.save()

		# Echo Success
		return HttpResponse("success")

	else:
		raise Http404

@ajax_request
def update_people(request):
	all_friends = Friend.objects.filter(exclude=0)
	ids = []
	for f in all_friends:
		ids.append(f.id)
	return {
		'ids' : ids
	}